@@ -7,8 +7,8 @@
 #++
 
 class Gem::Ext::ExtConfBuilder < Gem::Ext::Builder
-  def self.build(extension, dest_path, results, args = [], lib_dir = nil, extension_dir = Dir.pwd,
-    target_rbconfig = Gem.target_rbconfig, n_jobs: nil)
+  def self.build(extension, dest_path, results, args=[], lib_dir=nil, extension_dir=Dir.pwd,
+    target_rbconfig=Gem.target_rbconfig)
     require "fileutils"
     require "tempfile"
 
@@ -41,7 +41,7 @@ class Gem::Ext::ExtConfBuilder < Gem::Ext::Builder
 
       ENV["DESTDIR"] = nil
 
-      make dest_path, results, extension_dir, tmp_dest_relative, target_rbconfig: target_rbconfig, n_jobs: n_jobs
+      make dest_path, results, extension_dir, tmp_dest_relative, target_rbconfig: target_rbconfig
 
       full_tmp_dest = File.join(extension_dir, tmp_dest_relative)
 
@@ -60,15 +60,12 @@ class Gem::Ext::ExtConfBuilder < Gem::Ext::Builder
         destent.exist? || FileUtils.mv(ent.path, destent.path)
       end
 
+      make dest_path, results, extension_dir, tmp_dest_relative, ["clean"], target_rbconfig: target_rbconfig
     ensure
       ENV["DESTDIR"] = destdir
     end
 
     results
-  rescue Gem::Ext::Builder::NoMakefileError => error
-    results << error.message
-    results << "Skipping make for #{extension} as no Makefile was found."
-    # We are good, do not re-raise the error.
   ensure
     FileUtils.rm_rf tmp_dest if tmp_dest
   end
